@@ -34,7 +34,8 @@ class _LazyModule(_ModuleType):
     @classmethod
     def load(cls, pkg, mod, asname=None):
         if mod in _modules:
-            return _modules[pkg]
+            key = pkg if asname is None else mod
+            return _modules[key]
         else:
             return cls(pkg, mod, asname)
 
@@ -68,9 +69,10 @@ hashlib = _LazyModule.load('hashlib', 'hashlib')
 pathlib = _LazyModule.load('pathlib', 'pathlib')
 builtins = _LazyModule.load('builtins', 'builtins')
 subprocess = _LazyModule.load('subprocess', 'subprocess')
-from xonsh.platform import ON_WINDOWS
-
-RE_DASHF = re.compile(r'-F\s+(\w+)')
+xl = _LazyModule.load('xonsh', 'xonsh.lazyasd', 'xl')
+xp = _LazyModule.load('xonsh', 'xonsh.platform', 'xp')
+RE_DASHF = xl.LazyObject(lambda: re.compile(r'-F\s+(\w+)'),
+                         globals(), 'RE_DASHF')
 
 INITED = False
 
@@ -93,13 +95,6 @@ for ((i=0;i<${{#COMPREPLY[*]}};i++)) do echo ${{COMPREPLY[i]}}; done
 """
 
 
-if ON_WINDOWS:
-    from xonsh.platform import windows_bash_command
-    BASH_COMMAND = windows_bash_command()
-else:
-    BASH_COMMAND = 'bash'
-
-
 def update_bash_completion():
     global BASH_COMPLETE_FUNCS, BASH_COMPLETE_FILES, BASH_COMPLETE_HASH
     global CACHED_FUNCS, CACHED_FILES, CACHED_HASH, INITED
@@ -174,7 +169,7 @@ def complete_from_bash(prefix, line, begidx, endidx, ctx):
         end=endidx + 1, prefix=prefix, prev=shlex.quote(prev))
     try:
         out = subprocess.check_output(
-            [BASH_COMMAND], input=script, universal_newlines=True,
+            [xp.bash_command()], input=script, universal_newlines=True,
             stderr=subprocess.PIPE, env=builtins.__xonsh_env__.detype())
     except (subprocess.CalledProcessError, FileNotFoundError):
         out = ''
@@ -216,7 +211,7 @@ def _load_bash_complete_files():
     func_files = {}
     for line in out.splitlines():
         parts = line.split()
-        if ON_WINDOWS:
+        if xp.ON_WINDOWS:
             parts = [parts[0], ' '.join(parts[2:])]
         func_files[parts[0]] = parts[-1]
     BASH_COMPLETE_FILES = {
@@ -229,8 +224,9 @@ def _load_bash_complete_files():
 def _source_completions(source):
     try:
         return subprocess.check_output(
-            [BASH_COMMAND], input='\n'.join(source), universal_newlines=True,
-            env=builtins.__xonsh_env__.detype(), stderr=subprocess.DEVNULL)
+            [xp.bash_command()], input='\n'.join(source),
+            universal_newlines=True, env=builtins.__xonsh_env__.detype(),
+            stderr=subprocess.DEVNULL)
     except FileNotFoundError:
         return ''
 
@@ -295,18 +291,20 @@ def complete_completer(prefix, line, start, end, ctx):
 #
 # amalgamated re
 # amalgamated subprocess
-from xonsh.lazyasd import LazyObject, lazyobject
-
-PIP_RE = LazyObject(lambda: re.compile("pip(?:\d|\.)*"),
-                    globals(), 'PIP_RE')
-PIP_LIST_RE = LazyObject(lambda: re.compile("pip(?:\d|\.)* (?:uninstall|show)"),
-                         globals(), 'PIP_LIST_RE')
+# amalgamated xonsh.lazyasd
+PIP_RE = xl.LazyObject(lambda: re.compile("pip(?:\d|\.)*"),
+                       globals(), 'PIP_RE')
+PIP_LIST_RE = xl.LazyObject(lambda: re.compile("pip(?:\d|\.)* (?:uninstall|show)"),
+                            globals(), 'PIP_LIST_RE')
 
 
-@lazyobject
+@xl.lazyobject
 def ALL_COMMANDS():
-    help_text = str(subprocess.check_output(['pip', '--help'],
-                                            stderr=subprocess.DEVNULL))
+    try:
+        help_text = str(subprocess.check_output(['pip', '--help'],
+                                                stderr=subprocess.DEVNULL))
+    except FileNotFoundError:
+        return []
     commands = re.findall("  (\w+)  ", help_text)
     return [c for c in commands if c not in ['completion', 'help']]
 
@@ -318,7 +316,11 @@ def complete_pip(prefix, line, begidx, endidx, ctx):
                          (not PIP_RE.search(line)):
         return
     if PIP_LIST_RE.search(line):
-        items = subprocess.check_output(['pip', 'list'], stderr=subprocess.DEVNULL)
+        try:
+            items = subprocess.check_output(['pip', 'list'],
+                                            stderr=subprocess.DEVNULL)
+        except FileNotFoundError:
+            return set()
         items = items.decode('utf-8').splitlines()
         return set(i.split()[0] for i in items)
 
@@ -334,6 +336,7 @@ def complete_pip(prefix, line, begidx, endidx, ctx):
 #
 # tools
 #
+"""Xonsh completer tools."""
 # amalgamated builtins
 textwrap = _LazyModule.load('textwrap', 'textwrap')
 def _filter_normal(s, x):
@@ -369,14 +372,15 @@ def justify(s, max_length, left_pad=0):
 # _aliases
 #
 # amalgamated builtins
-from collections import OrderedDict
-
+collections = _LazyModule.load('collections', 'collections')
+# amalgamated xonsh.lazyasd
 # amalgamated xonsh.completers.tools
-VALID_ACTIONS = frozenset({'add', 'remove', 'list'})
+VALID_ACTIONS = xl.LazyObject(lambda: frozenset({'add', 'remove', 'list'}),
+                              globals(), 'VALID_ACTIONS')
 
 
 def _add_one_completer(name, func, loc='end'):
-    new = OrderedDict()
+    new = collections.OrderedDict()
     if loc == 'start':
         new[name] = func
         for (k, v) in builtins.__xonsh_completers__.items():
@@ -485,25 +489,22 @@ def completer_alias(args, stdin=None):
         func = _list_completers
     return func(args[1:], stdin=stdin)
 
-COMPLETER_LIST_HELP_STR = """
-completer list: list the active completers, in order
+COMPLETER_LIST_HELP_STR = """completer list: ordered list the active completers
 
 Usage:
     completer remove
-""".lstrip()
+"""
 
-COMPLETER_REMOVE_HELP_STR = """
-completer remove: removes a completer from xonsh
+COMPLETER_REMOVE_HELP_STR = """completer remove: removes a completer from xonsh
 
 Usage:
     completer remove NAME
 
 NAME is a unique name of a completer (run "completer list" to see the current
      completers in order)
-""".lstrip()
+"""
 
-COMPLETER_ADD_HELP_STR = """
-completer add: adds a new completer to xonsh
+COMPLETER_ADD_HELP_STR = """completer add: adds a new completer to xonsh
 
 Usage:
     completer add NAME FUNC [POS]
@@ -540,15 +541,15 @@ POS (optional) is a position into the list of completers at which the new
                  be added before the completer named KEY
 
      If POS is not provided, the default value is "start"
-""".lstrip()
+"""
 
 #
 # commands
 #
 # amalgamated os
 # amalgamated builtins
-from xonsh.tools import executables_in
-# amalgamated from xonsh.platform import ON_WINDOWS
+xt = _LazyModule.load('xonsh', 'xonsh.tools', 'xt')
+# amalgamated xonsh.platform
 # amalgamated xonsh.completers.tools
 SKIP_TOKENS = {'sudo', 'time', 'timeit', 'which', 'showcmd', 'man'}
 
@@ -561,14 +562,13 @@ def complete_command(cmd, line, start, end, ctx):
     out = {s + space
            for s in builtins.__xonsh_commands_cache__
            if get_filter_function()(s, cmd)}
-    if ON_WINDOWS:
-        out |= {i
-                for i in executables_in('.')
+    if xp.ON_WINDOWS:
+        out |= {i for i in xt.executables_in('.')
                 if i.startswith(cmd)}
     base = os.path.basename(cmd)
     if os.path.isdir(base):
         out |= {os.path.join(base, i)
-                for i in executables_in(base)
+                for i in xt.executables_in(base)
                 if i.startswith(cmd)}
     return out
 
@@ -596,7 +596,6 @@ def complete_skipper(cmd, line, start, end, ctx):
                          start - len(first) - 1,
                          end - len(first) - 1,
                          ctx)
-
     else:
         return set()
 
@@ -608,12 +607,20 @@ def complete_skipper(cmd, line, start, end, ctx):
 # amalgamated pickle
 # amalgamated builtins
 # amalgamated subprocess
+# amalgamated xonsh.lazyasd
 # amalgamated xonsh.completers.tools
 OPTIONS = None
 OPTIONS_PATH = None
 
-SCRAPE_RE = re.compile(r'^(?:\s*(?:-\w|--[a-z0-9-]+)[\s,])+', re.M)
-INNER_OPTIONS_RE = re.compile(r'-\w|--[a-z0-9-]+')
+
+@xl.lazyobject
+def SCRAPE_RE():
+    return re.compile(r'^(?:\s*(?:-\w|--[a-z0-9-]+)[\s,])+', re.M)
+
+
+@xl.lazyobject
+def INNER_OPTIONS_RE():
+    return re.compile(r'-\w|--[a-z0-9-]+')
 
 
 def complete_from_man(prefix, line, start, end, ctx):
@@ -656,25 +663,25 @@ def complete_from_man(prefix, line, start, end, ctx):
 # path
 #
 # amalgamated os
-# amalgamated re
 ast = _LazyModule.load('ast', 'ast')
 # amalgamated builtins
-# amalgamated from xonsh.platform import ON_WINDOWS
-from xonsh.tools import (subexpr_from_unbalanced, get_sep,
-                         check_for_partial_string, RE_STRING_START,
-                         iglobpath, levenshtein)
-
+# amalgamated xonsh.tools
+# amalgamated xonsh.platform
+# amalgamated xonsh.lazyasd
 # amalgamated xonsh.completers.tools
-CHARACTERS_NEED_QUOTES = ' `\t\r\n${}*()"\',?&'
-if ON_WINDOWS:
-    CHARACTERS_NEED_QUOTES += '%'
+@xl.lazyobject
+def CHARACTERS_NEED_QUOTES():
+    cnq = ' `\t\r\n${}*()"\',?&'
+    if xp.ON_WINDOWS:
+        cnq += '%'
+    return cnq
 
 
 def _path_from_partial_string(inp, pos=None):
     if pos is None:
         pos = len(inp)
     partial = inp[:pos]
-    startix, endix, quote = check_for_partial_string(partial)
+    startix, endix, quote = xt.check_for_partial_string(partial)
     _post = ""
     if startix is None:
         return None
@@ -688,7 +695,7 @@ def _path_from_partial_string(inp, pos=None):
             else:
                 return None
         string = partial[startix:endix]
-    end = re.sub(RE_STRING_START, '', quote)
+    end = xt.RE_STRING_START.sub('', quote)
     _string = string
     if not _string.endswith(end):
         _string = _string + end
@@ -709,19 +716,17 @@ def _normpath(p):
     and '/' at the end. On windows it does the same with backslashes
     """
     initial_dotslash = p.startswith(os.curdir + os.sep)
-    initial_dotslash |= (ON_WINDOWS and p.startswith(os.curdir + os.altsep))
+    initial_dotslash |= (xp.ON_WINDOWS and p.startswith(os.curdir + os.altsep))
     p = p.rstrip()
     trailing_slash = p.endswith(os.sep)
-    trailing_slash |= (ON_WINDOWS and p.endswith(os.altsep))
+    trailing_slash |= (xp.ON_WINDOWS and p.endswith(os.altsep))
     p = os.path.normpath(p)
     if initial_dotslash and p != '.':
         p = os.path.join(os.curdir, p)
     if trailing_slash:
         p = os.path.join(p, '')
-
-    if ON_WINDOWS and builtins.__xonsh_env__.get('FORCE_POSIX_PATHS'):
+    if xp.ON_WINDOWS and builtins.__xonsh_env__.get('FORCE_POSIX_PATHS'):
         p = p.replace(os.sep, os.altsep)
-
     return p
 
 
@@ -745,7 +750,7 @@ def _env(prefix):
 
 
 def _dots(prefix):
-    slash = get_sep()
+    slash = xt.get_sep()
     if slash == '\\':
         slash = ''
     if prefix in {'', '.'}:
@@ -763,8 +768,8 @@ def _add_cdpaths(paths, prefix):
     glob_sorted = env.get('GLOB_SORTED')
     for cdp in env.get('CDPATH'):
         test_glob = os.path.join(cdp, prefix) + '*'
-        for s in iglobpath(test_glob, ignore_case=(not csc),
-                           sort_result=glob_sorted):
+        for s in xt.iglobpath(test_glob, ignore_case=(not csc),
+                              sort_result=glob_sorted):
             if os.path.isdir(s):
                 paths.add(os.path.basename(s))
 
@@ -784,7 +789,7 @@ def _quote_paths(paths, start, end):
     space = ' '
     backslash = '\\'
     double_backslash = '\\\\'
-    slash = get_sep()
+    slash = xt.get_sep()
     orig_start = start
     orig_end = end
     for s in paths:
@@ -821,9 +826,9 @@ def _joinpath(path):
     elif len(path) == 0:
         return ''
     elif path == ('',):
-        return get_sep()
+        return xt.get_sep()
     elif path[0] == '':
-        return get_sep() + _normpath(os.path.join(*path))
+        return xt.get_sep() + _normpath(os.path.join(*path))
     else:
         return _normpath(os.path.join(*path))
 
@@ -832,7 +837,7 @@ def _splitpath(path):
     # convert a path into an intermediate tuple representation
     # if this tuple starts with '', it means that the path was an absolute path
     path = _normpath(path)
-    if path.startswith(get_sep()):
+    if path.startswith(xt.get_sep()):
         pre = ('', )
     else:
         pre = ()
@@ -882,7 +887,7 @@ def _expand_one(sofar, nextone, csc):
     glob_sorted = builtins.__xonsh_env__.get('GLOB_SORTED')
     for i in sofar:
         _glob = os.path.join(_joinpath(i), '*') if i is not None else '*'
-        for j in iglobpath(_glob, sort_result=glob_sorted):
+        for j in xt.iglobpath(_glob, sort_result=glob_sorted):
             j = os.path.basename(j)
             if subsequence_match(j, nextone, csc):
                 out.add((i or ()) + (j, ))
@@ -906,8 +911,8 @@ def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
     env = builtins.__xonsh_env__
     csc = env.get('CASE_SENSITIVE_COMPLETIONS')
     glob_sorted = env.get('GLOB_SORTED')
-    for s in iglobpath(prefix + '*', ignore_case=(not csc),
-                       sort_result=glob_sorted):
+    for s in xt.iglobpath(prefix + '*', ignore_case=(not csc),
+                          sort_result=glob_sorted):
         paths.add(s)
     if len(paths) == 0 and env.get('SUBSEQUENCE_PATH_COMPLETION'):
         # this block implements 'subsequence' matching, similar to fish and zsh.
@@ -927,9 +932,10 @@ def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
             paths |= {_joinpath(i) for i in matches_so_far}
     if len(paths) == 0 and env.get('FUZZY_PATH_COMPLETION'):
         threshold = env.get('SUGGEST_THRESHOLD')
-        for s in iglobpath(os.path.dirname(prefix) + '*', ignore_case=(not csc),
-                           sort_result=glob_sorted):
-            if levenshtein(prefix, s, threshold) < threshold:
+        for s in xt.iglobpath(os.path.dirname(prefix) + '*',
+                              ignore_case=(not csc),
+                              sort_result=glob_sorted):
+            if xt.levenshtein(prefix, s, threshold) < threshold:
                 paths.add(s)
     if tilde in prefix:
         home = os.path.expanduser(tilde)
@@ -952,27 +958,33 @@ def complete_dir(prefix, line, start, end, ctx, cdpath=False):
 #
 # python
 #
+"""Completers for Python code"""
 # amalgamated re
 sys = _LazyModule.load('sys', 'sys')
 inspect = _LazyModule.load('inspect', 'inspect')
 # amalgamated builtins
 importlib = _LazyModule.load('importlib', 'importlib')
-from collections.abc import Iterable
-
-# amalgamated from xonsh.tools import subexpr_from_unbalanced, get_sep, check_for_partial_string, RE_STRING_START
+abc = _LazyModule.load('collections', 'collections.abc', 'abc')
+# amalgamated xonsh.tools
+# amalgamated xonsh.lazyasd
 # amalgamated xonsh.completers.tools
-RE_ATTR = re.compile(r'([^\s\(\)]+(\.[^\s\(\)]+)*)\.(\w*)$')
+@xl.lazyobject
+def RE_ATTR():
+    return re.compile(r'([^\s\(\)]+(\.[^\s\(\)]+)*)\.(\w*)$')
 
-XONSH_TOKENS = {
-    'and ', 'as ', 'assert ', 'break', 'class ', 'continue', 'def ', 'del ',
-    'elif ', 'else', 'except ', 'finally:', 'for ', 'from ', 'global ',
-    'import ', 'if ', 'in ', 'is ', 'lambda ', 'nonlocal ', 'not ', 'or ',
-    'pass', 'raise ', 'return ', 'try:', 'while ', 'with ', 'yield ', '+', '-',
-    '/', '//', '%', '**', '|', '&', '~', '^', '>>', '<<', '<', '<=', '>', '>=',
-    '==', '!=', '->', '=', '+=', '-=', '*=', '/=', '%=', '**=', '>>=', '<<=',
-    '&=', '^=', '|=', '//=', ',', ';', ':', '?', '??', '$(', '${', '$[', '..',
-    '...', '![', '!(', '@(', '@$(', '@'
-}
+
+@xl.lazyobject
+def XONSH_TOKENS():
+    return {
+        'and ', 'as ', 'assert ', 'break', 'class ', 'continue', 'def ', 'del ',
+        'elif ', 'else', 'except ', 'finally:', 'for ', 'from ', 'global ',
+        'import ', 'if ', 'in ', 'is ', 'lambda ', 'nonlocal ', 'not ', 'or ',
+        'pass', 'raise ', 'return ', 'try:', 'while ', 'with ', 'yield ', '+',
+        '-', '/', '//', '%', '**', '|', '&', '~', '^', '>>', '<<', '<', '<=',
+        '>', '>=', '==', '!=', '->', '=', '+=', '-=', '*=', '/=', '%=', '**=',
+        '>>=', '<<=', '&=', '^=', '|=', '//=', ',', ';', ':', '?', '??', '$(',
+        '${', '$[', '..', '...', '![', '!(', '@(', '@$(', '@'
+        }
 
 
 def complete_python(prefix, line, start, end, ctx):
@@ -1012,9 +1024,9 @@ def attr_complete(prefix, ctx, filter_func):
     if m is None:
         return attrs
     expr, attr = m.group(1, 3)
-    expr = subexpr_from_unbalanced(expr, '(', ')')
-    expr = subexpr_from_unbalanced(expr, '[', ']')
-    expr = subexpr_from_unbalanced(expr, '{', '}')
+    expr = xt.subexpr_from_unbalanced(expr, '(', ')')
+    expr = xt.subexpr_from_unbalanced(expr, '[', ']')
+    expr = xt.subexpr_from_unbalanced(expr, '{', '}')
     _ctx = None
     xonsh_safe_eval = builtins.__xonsh_execer__.eval
     try:
@@ -1041,7 +1053,7 @@ def attr_complete(prefix, ctx, filter_func):
         a = getattr(val, opt)
         if callable(a):
             rpl = opt + '('
-        elif isinstance(a, Iterable):
+        elif isinstance(a, abc.Iterable):
             rpl = opt + '['
         else:
             rpl = opt
@@ -1105,7 +1117,6 @@ def complete_base(prefix, line, start, end, ctx):
 #
 # dirs
 #
-# amalgamated os
 # amalgamated xonsh.completers.man
 # amalgamated xonsh.completers.path
 def complete_cd(prefix, line, start, end, ctx):
@@ -1132,7 +1143,7 @@ def complete_rmdir(prefix, line, start, end, ctx):
 # init
 #
 """Constructor for xonsh completer objects."""
-# amalgamated from collections import OrderedDict
+# amalgamated collections
 # amalgamated xonsh.completers.pip
 # amalgamated xonsh.completers.man
 # amalgamated xonsh.completers.bash
@@ -1143,8 +1154,8 @@ def complete_rmdir(prefix, line, start, end, ctx):
 # amalgamated xonsh.completers.commands
 # amalgamated xonsh.completers.completer
 def default_completers():
-    """Creates a copy of the default completers.    """
-    return OrderedDict([
+    """Creates a copy of the default completers."""
+    return collections.OrderedDict([
         ('python_mode', complete_python_mode),
         ('base', complete_base),
         ('completer', complete_completer),
