@@ -1,12 +1,11 @@
 # -*- coding: utf-8 -*-
 """Implements the xonsh executer."""
-import re
 import sys
 import types
 import inspect
 import builtins
 import warnings
-from collections import Mapping
+import collections.abc as abc
 
 from xonsh.ast import CtxAwareTransformer
 from xonsh.parser import Parser
@@ -81,7 +80,7 @@ class Execer(object):
         # it also is valid as a subprocess line.
         if ctx is None:
             ctx = set()
-        elif isinstance(ctx, Mapping):
+        elif isinstance(ctx, abc.Mapping):
             ctx = set(ctx.keys())
         tree = self.ctxtransformer.ctxvisit(tree, input, ctx, mode=mode)
         return tree
@@ -111,7 +110,7 @@ class Execer(object):
         return code
 
     def eval(self, input, glbs=None, locs=None, stacklevel=2,
-                transform=True):
+             transform=True):
         """Evaluates (and returns) xonsh code."""
         if isinstance(input, types.CodeType):
             code = input
@@ -127,7 +126,7 @@ class Execer(object):
         return eval(code, glbs, locs)
 
     def exec(self, input, mode='exec', glbs=None, locs=None, stacklevel=2,
-                transform=True):
+             transform=True):
         """Execute xonsh code."""
         if isinstance(input, types.CodeType):
             code = input
@@ -205,7 +204,7 @@ class Execer(object):
                         # or for some other syntax error
                         raise original_error
                 elif sbpline[last_error_col:].startswith('![![') or \
-                     sbpline.lstrip().startswith('![!['):
+                        sbpline.lstrip().startswith('![!['):
                     # if we have already wrapped this in subproc tokens
                     # and it still doesn't work, adding more won't help
                     # anything
