@@ -4,17 +4,17 @@ import builtins
 
 from prompt_toolkit.enums import DEFAULT_BUFFER
 from prompt_toolkit.filters import (Condition, Filter, IsMultiline,
-                                    HasSelection)
+                                    HasSelection, EmacsInsertMode,
+                                    ViInsertMode)
 from prompt_toolkit.keys import Keys
 from xonsh.aliases import xonsh_exit
 from xonsh.tools import ON_WINDOWS
 
 env = builtins.__xonsh_env__
-indent_ = env.get('INDENT')
 DEDENT_TOKENS = frozenset(['raise', 'return', 'pass', 'break', 'continue'])
 
 
-def carriage_return(b, cli):
+def carriage_return(b, cli, *, autoindent=True):
     """
     Preliminary parser to determine if 'Enter' key should send command to
     the xonsh parser for execution or should insert a newline for continued
@@ -28,35 +28,37 @@ def carriage_return(b, cli):
     - Any text exists below cursor position (relevant when editing previous
     multiline blocks)
     """
+    doc = b.document
+    at_end_of_line = _is_blank(doc.current_line_after_cursor)
+    current_line_blank = _is_blank(doc.current_line)
 
-    at_end_of_line = _is_blank(b.document.current_line_after_cursor)
-    current_line_blank = _is_blank(b.document.current_line)
+    indent = env.get('INDENT') if autoindent else ''
 
     # indent after a colon
-    if (b.document.current_line_before_cursor.strip().endswith(':') and
+    if (doc.current_line_before_cursor.strip().endswith(':') and
             at_end_of_line):
-        b.newline()
-        b.insert_text(indent_, fire_event=False)
+        b.newline(copy_margin=autoindent)
+        b.insert_text(indent, fire_event=False)
     # if current line isn't blank, check dedent tokens
     elif (not current_line_blank and
-            b.document.current_line.split(maxsplit=1)[0] in DEDENT_TOKENS and
-            b.document.line_count > 1):
-        b.newline(copy_margin=True)
-        _ = b.delete_before_cursor(count=len(indent_))
-    elif (not b.document.on_first_line and
+            doc.current_line.split(maxsplit=1)[0] in DEDENT_TOKENS and
+            doc.line_count > 1):
+        b.newline(copy_margin=autoindent)
+        b.delete_before_cursor(count=len(indent))
+    elif (not doc.on_first_line and
             not current_line_blank):
-        b.newline(copy_margin=True)
-    elif (b.document.char_before_cursor == '\\' and
+        b.newline(copy_margin=autoindent)
+    elif (doc.char_before_cursor == '\\' and
             not (not builtins.__xonsh_env__.get('FORCE_POSIX_PATHS')
-                and ON_WINDOWS)):
-        b.newline()
-    elif (b.document.find_next_word_beginning() is not None and
+                 and ON_WINDOWS)):
+        b.newline(copy_margin=autoindent)
+    elif (doc.find_next_word_beginning() is not None and
             (any(not _is_blank(i)
                  for i
-                 in b.document.lines_from_current[1:]))):
-        b.newline(copy_margin=True)
-    elif not current_line_blank and not can_compile(b.document.text):
-        b.newline()
+                 in doc.lines_from_current[1:]))):
+        b.newline(copy_margin=autoindent)
+    elif not current_line_blank and not can_compile(doc.text):
+        b.newline(copy_margin=autoindent)
     else:
         b.accept_action.validate_and_handle(cli, b)
 
@@ -73,6 +75,7 @@ class TabShouldInsertIndentFilter(Filter):
 
         return bool(before_cursor.isspace())
 
+
 class BeginningOfLine(Filter):
     """
     Check if cursor is at beginning of a line other than the first line
@@ -84,6 +87,7 @@ class BeginningOfLine(Filter):
         return bool(len(before_cursor) == 0
                     and not cli.current_buffer.document.on_first_line)
 
+
 class EndOfLine(Filter):
     """
     Check if cursor is at the end of a line other than the last line
@@ -102,8 +106,11 @@ class EndOfLine(Filter):
 def ctrl_d_condition(cli):
     """ Ctrl-D binding is only active when the default buffer is selected
     and empty. """
-    return (cli.current_buffer_name == DEFAULT_BUFFER and
-            not cli.current_buffer.text)
+    if builtins.__xonsh_env__.get("IGNOREEOF"):
+        raise EOFError
+    else:
+        return (cli.current_buffer_name == DEFAULT_BUFFER and
+                not cli.current_buffer.text)
 
 
 def can_compile(src):
@@ -127,6 +134,7 @@ def load_xonsh_bindings(key_bindings_manager):
     """
     handle = key_bindings_manager.registry.add_binding
     has_selection = HasSelection()
+    insert_mode = ViInsertMode() | EmacsInsertMode()
 
     @handle(Keys.Tab, filter=TabShouldInsertIndentFilter())
     def _(event):
@@ -175,5 +183,33 @@ def load_xonsh_bindings(key_bindings_manager):
         b.cursor_left(count=abs(relative_begin_index))
         b.cursor_down(count=1)
 
+    @handle(Keys.ControlI, filter=insert_mode)
+    def generate_completions(event):
+        """
+        Tab-completion: where the first tab completes the common suffix and the
+        second tab lists all the completions.
+
+        Notes
+        -----
+        This method was forked from the mainline prompt-toolkit repo.
+        Copyright (c) 2014, Jonathan Slenders, All rights reserved.
+        """
+        b = event.current_buffer
+
+        def second_tab():
+            if b.complete_state:
+                b.complete_next()
+            else:
+                event.cli.start_completion(select_first=False)
+
+        # On the second tab-press, or when already navigating through
+        # completions.
+        if event.is_repeat or b.complete_state:
+            second_tab()
+        else:
+            event.cli.start_completion(insert_common_part=True,
+                                       select_first=False)
+
+
 def _is_blank(l):
     return len(l.strip()) == 0
