@@ -12,29 +12,26 @@ are included from the IPython project.  The IPython project is:
 """
 import os
 import sys
+import cmd
 import time
 import select
 import builtins
 import importlib
-from cmd import Cmd
-from threading import Thread
-from collections import deque
+import threading
+import collections
 
 from xonsh.lazyjson import LazyJSON
 from xonsh.lazyasd import LazyObject
 from xonsh.base_shell import BaseShell
-from xonsh.ansi_colors import partial_color_format, color_style_names, color_style
+from xonsh.ansi_colors import ansi_partial_color_format, ansi_color_style_names, ansi_color_style
 from xonsh.environ import partial_format_prompt, multiline_prompt
 from xonsh.tools import print_exception
 from xonsh.platform import ON_WINDOWS, ON_CYGWIN, ON_DARWIN
+from xonsh.lazyimps import pygments, pyghooks
 
-pygments = LazyObject(lambda: importlib.import_module('pygments'),
-                      globals(), 'pygments')
-terminal256 = LazyObject(lambda: importlib.import_module(
-                                    'pygments.formatters.terminal256'),
-                      globals(), 'terminal')
-pyghooks = LazyObject(lambda: importlib.import_module('xonsh.pyghooks'),
-                      globals(), 'pyghooks')
+terminal256 = LazyObject(
+    lambda: importlib.import_module('pygments.formatters.terminal256'),
+    globals(), 'terminal')
 
 readline = None
 RL_COMPLETION_SUPPRESS_APPEND = RL_LIB = RL_STATE = None
@@ -78,7 +75,7 @@ def setup_readline():
             RL_COMPLETION_SUPPRESS_APPEND = None
         try:
             RL_STATE = ctypes.c_int.in_dll(lib, 'rl_readline_state')
-        except:
+        except Exception:
             pass
         RL_CAN_RESIZE = hasattr(lib, 'rl_reset_screen_size')
     env = builtins.__xonsh_env__
@@ -95,16 +92,16 @@ def setup_readline():
     # as discussed at http://stackoverflow.com/a/7116997
     if uses_libedit and ON_DARWIN:
         readline.parse_and_bind("bind ^I rl_complete")
-        print('\n'.join(['', "*"*78,
-            "libedit detected - readline will not be well behaved, including but not limited to:",
-            "   * crashes on tab completion",
-            "   * incorrect history navigation",
-            "   * corrupting long-lines",
-            "   * failure to wrap or indent lines properly",
-            "",
-            "It is highly recommended that you install gnureadline, which is installable with:",
-            "     pip install gnureadline",
-            "*"*78]), file=sys.stderr)
+        print('\n'.join(['', "*" * 78,
+                         "libedit detected - readline will not be well behaved, including but not limited to:",
+                         "   * crashes on tab completion",
+                         "   * incorrect history navigation",
+                         "   * corrupting long-lines",
+                         "   * failure to wrap or indent lines properly",
+                         "",
+                         "It is highly recommended that you install gnureadline, which is installable with:",
+                         "     pip install gnureadline",
+                         "*" * 78]), file=sys.stderr)
     else:
         readline.parse_and_bind("tab: complete")
     # try to load custom user settings
@@ -160,7 +157,7 @@ def fix_readline_state_after_ctrl_c():
             _q = readline.rl.mode.process_keyevent_queue
             if len(_q) > 1:
                 _q.pop()
-        except:
+        except Exception:
             pass
     if RL_STATE is None:
         return
@@ -203,16 +200,20 @@ def rl_variable_value(variable):
 
 def _insert_text_func(s, readline):
     """Creates a function to insert text via readline."""
+
     def inserter():
         readline.insert_text(s)
         readline.redisplay()
+
     return inserter
 
 
-DEDENT_TOKENS = frozenset(['raise', 'return', 'pass', 'break', 'continue'])
+DEDENT_TOKENS = LazyObject(lambda: frozenset(['raise', 'return', 'pass',
+                                              'break', 'continue']),
+                           globals(), 'DEDENT_TOKENS')
 
 
-class ReadlineShell(BaseShell, Cmd):
+class ReadlineShell(BaseShell, cmd.Cmd):
     """The readline based xonsh shell."""
 
     def __init__(self, completekey='tab', stdin=None, stdout=None, **kwargs):
@@ -224,7 +225,7 @@ class ReadlineShell(BaseShell, Cmd):
         self._current_indent = ''
         self._current_prompt = ''
         self._force_hide = None
-        self.cmdqueue = deque()
+        self.cmdqueue = collections.deque()
 
     def __del__(self):
         teardown_readline()
@@ -254,6 +255,7 @@ class ReadlineShell(BaseShell, Cmd):
         rl_completion_suppress_append()  # this needs to be called each time
         _rebind_case_sensitive_completions()
 
+        line = builtins.aliases.expand_alias(line)
         mline = line.partition(' ')[2]
         offs = len(mline) - len(text)
         if self.completer is None:
@@ -333,7 +335,7 @@ class ReadlineShell(BaseShell, Cmd):
             if intro is not None:
                 self.intro = intro
             if self.intro:
-                self.stdout.write(str(self.intro)+"\n")
+                self.stdout.write(str(self.intro) + "\n")
             stop = None
             while not stop:
                 line = None
@@ -343,7 +345,7 @@ class ReadlineShell(BaseShell, Cmd):
                     exec_now = line.endswith('\n')
                 if self.use_rawinput and not exec_now:
                     inserter = None if line is None \
-                                    else _insert_text_func(line, readline)
+                        else _insert_text_func(line, readline)
                     if inserter is not None:
                         readline.set_pre_input_hook(inserter)
                     try:
@@ -402,7 +404,6 @@ class ReadlineShell(BaseShell, Cmd):
             # This is needed to support some system where line-wrapping doesn't
             # work. This is a bug in upstream Python, or possibly readline.
             RL_LIB.rl_reset_screen_size()
-        #return super().prompt
         if self.need_more_lines:
             if self.mlprompt is None:
                 try:
@@ -418,8 +419,8 @@ class ReadlineShell(BaseShell, Cmd):
         except Exception:  # pylint: disable=broad-except
             print_exception()
         hide = True if self._force_hide is None else self._force_hide
-        p = partial_color_format(p, style=env.get('XONSH_COLOR_STYLE'),
-                                 hide=hide)
+        p = ansi_partial_color_format(p, style=env.get('XONSH_COLOR_STYLE'),
+                                      hide=hide)
         self._current_prompt = p
         self.settitle()
         return p
@@ -429,8 +430,8 @@ class ReadlineShell(BaseShell, Cmd):
         codes.
         """
         hide = hide if self._force_hide is None else self._force_hide
-        return partial_color_format(string, hide=hide,
-                    style=builtins.__xonsh_env__.get('XONSH_COLOR_STYLE'))
+        return ansi_partial_color_format(string, hide=hide,
+                                         style=builtins.__xonsh_env__.get('XONSH_COLOR_STYLE'))
 
     def print_color(self, string, hide=False, **kwargs):
         if isinstance(string, str):
@@ -446,16 +447,15 @@ class ReadlineShell(BaseShell, Cmd):
 
     def color_style_names(self):
         """Returns an iterable of all available style names."""
-        return color_style_names()
+        return ansi_color_style_names()
 
     def color_style(self):
         """Returns the current color map."""
-        style = style=builtins.__xonsh_env__.get('XONSH_COLOR_STYLE')
-        return color_style(style=style)
+        style = style = builtins.__xonsh_env__.get('XONSH_COLOR_STYLE')
+        return ansi_color_style(style=style)
 
 
-class ReadlineHistoryAdder(Thread):
-
+class ReadlineHistoryAdder(threading.Thread):
     def __init__(self, wait_for_gc=True, *args, **kwargs):
         """Thread responsible for adding inputs from history to the current readline
         instance. May wait for the history garbage collector to finish.
@@ -478,8 +478,8 @@ class ReadlineHistoryAdder(Thread):
         for _, _, f in files:
             try:
                 lj = LazyJSON(f, reopen=False)
-                for cmd in lj['cmds']:
-                    inp = cmd['inp'].splitlines()
+                for command in lj['cmds']:
+                    inp = command['inp'].splitlines()
                     for line in inp:
                         if line == 'EOF':
                             continue
